--[[
Author:        David Szybala
Website:       www.davidszybala.com
Alt Website:   www.twistedgerbil.com
Program:       Render_Selected_Savers.lua
Date:          5/9/18
Description:   This script will render only the selected savers
               over user defined frame ranges
]]


function get_frame_range()
    --[[Creates a dialog asking the user for the range to render]]
    local ui_controls = {}
    local range = composition:GetAttrs("COMPN_GlobalStart") .. " - " .. composition:GetAttrs("COMPN_GlobalEnd")
    ui_controls[1] = {"frame_range", Name = "Frame Range (e.g. 1-10, 32, 50-70x2)", "Text", Lines = 1, Default = range}
    local ui = composition:AskUser("Render Selected Savers", ui_controls)
    return ui
end


function split_string(search_string, split_char)
    --[[Splits 'search_string' by 'split_char' and returns an array]]
    local tokens = {}

    for split in string.gmatch(string.lower(search_string), ("([^" .. split_char .. "]+)")) do
        table.insert(tokens, split)
    end

    return tokens
end


function convert_to_integers(the_array)
    --[[Converts the string array to an integer array. Returns true on success ]]
    for i,v in ipairs(the_array) do
        the_array[i] = tonumber(v)
        
        if the_array[i] == nil then
            return false
        end
    end
    
    return true
end


function get_table_length(the_table)
    --[[Returns the length of the table]]
    local counter = 0
    
    for _ in pairs(the_table) do
        counter = counter + 1
    end
    
    return counter
end


function render_saver(saver_node, frame_ranges)
    --[[Render the 'saver_node' across all 'frame_ranges' ]]
    for i,v in ipairs(frame_ranges) do
        settings = {Tool = saver_node, Wait = true, Start = v[1], End = v[1], StepRender = true, Steps = 1}
        length = get_table_length(v)
        
        if length > 1 then
            settings["End"] = v[2]
        end
        
        if length > 2 then
            settings["Steps"] = v[3]
        end
        
        composition:Render(settings)    
    end
end


function main()
    --[[Main entry point. Renders the selected saver nodes]]
    composition:Lock()
    local selected_savers = composition:GetToolList(true, "Saver")
    
    if get_table_length(selected_savers) == 0 then
        print("ERROR: No savers selected")
        return false
    end

    ui_results = get_frame_range()

    if ui_results == nil then
        print("WARNING: User cancelled dialog")
        return false
    else
        --convert the frame ranges
        local all_ranges = split_string(ui_results["frame_range"], ",")
        local frame_ranges = {}
    
        for i,v in ipairs(all_ranges) do
            table.insert(frame_ranges, split_string(v, "-x"))
        end
        
        --validate the ranges
        for i,v in ipairs(frame_ranges) do
            length = get_table_length(v)
            
            if length > 3 then
                print("ERROR: Unknown frame range: " .. all_ranges[i])
                return false
            end
            
            result = convert_to_integers(v)
            
            if not result then
                print("ERROR: Could not convert frame range: " .. all_ranges[i])
                return false
            end
        end
        
        --store saver states, then disable all savers for safety
        savers = composition:GetToolList(false, "Saver")
        saver_states = {}
        for k,v in pairs(savers) do
            table.insert(saver_states, v:GetAttrs("TOOLB_PassThrough"))
            v:SetAttrs({TOOLB_PassThrough = true})
        end
        
        --render the selected savers
        for k,v in pairs(selected_savers) do
            v:SetAttrs({TOOLB_PassThrough = false})
            render_saver(v, frame_ranges)
        end
        
        --restore the saver states
        for i,v in ipairs(saver_states) do
            savers[i]:SetAttrs({TOOLB_PassThrough = saver_states[i]})
        end
    end
    
    composition:Unlock()
    print("INFO: Rendering complete")
end


main()